/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2011-2024 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "multipliedFixedValueFvPatchField.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class Type>
Foam::multipliedFixedValueFvPatchField<Type>::multipliedFixedValueFvPatchField
(
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<Type>(p, iF, dict, false),
    multipliedValue_
    (
        Function1<Type>::New
        (
            "toBeScaledValue",
            this->db().time().userUnits(),
            iF.dimensions(),
            dict
        )
    ),
    scale_(dict.lookup<scalar>("scale", dimless))
{
    this->evaluate();
}


template<class Type>
Foam::multipliedFixedValueFvPatchField<Type>::multipliedFixedValueFvPatchField
(
    const multipliedFixedValueFvPatchField<Type>& ptf,
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const fieldMapper& mapper
)
:
    fixedValueFvPatchField<Type>(ptf, p, iF, mapper, false), // Don't map
    multipliedValue_(ptf.multipliedValue_, false),
    scale_(ptf.scale_)
{
    // Evaluate since value not mapped
    this->evaluate();
}


template<class Type>
Foam::multipliedFixedValueFvPatchField<Type>::multipliedFixedValueFvPatchField
(
    const multipliedFixedValueFvPatchField<Type>& ptf,
    const DimensionedField<Type, volMesh>& iF
)
:
    fixedValueFvPatchField<Type>(ptf, iF),
    multipliedValue_(ptf.multipliedValue_, false),
    scale_(ptf.scale_)
{
    // Evaluate the profile if defined
    if (ptf.multipliedValue_.valid())
    {
        this->evaluate();
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class Type>
void Foam::multipliedFixedValueFvPatchField<Type>::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }
    
    fvPatchField<Type>::operator==
    (
        multipliedValue_->value(this->db().time().value())*scale_
    );

    fixedValueFvPatchField<Type>::updateCoeffs();
}


template<class Type>
void Foam::multipliedFixedValueFvPatchField<Type>::write(Ostream& os) const
{
    fvPatchField<Type>::write(os);
    writeEntry
    (
        os,
        this->db().time().userUnits(),
        this->internalField().dimensions(),
        multipliedValue_()
    );
    writeEntry(os, "scale", scale_);
    writeEntry(os, "value", *this);
}


// ************************************************************************* //
